unit JS_GLBW;


Interface

Uses
 js_sub;

Type
 ParGlbwMod_T = Array [1..6] of Real;

Var
ParGlbwMod:        ParGlbwMod_T;


function  GlbwMod(x:Real):Real;

function  DGlbwnachz1(x:Real):Real;
function  DGlbwnachg1(x:Real):Real;
function  DGlbwnachy1(x:Real):Real;
function  DGlbwnachy2(x:Real):Real;
function  DGlbwnachg2(x:Real):Real;
function  DGlbwnachz2(x:Real):Real;

procedure NablaGlbw(x:Real; Var GradvektGlbw:ParGlbwMod_T);


Implementation

function z1:Real; begin z1:=ParGlbwMod[1]; end;
function g1:Real; begin g1:=ParGlbwMod[2]; end;
function y1:Real; begin y1:=ParGlbwMod[3]; end;
function y2:Real; begin y2:=ParGlbwMod[4]; end;
function g2:Real; begin g2:=ParGlbwMod[5]; end;
function z2:Real; begin z2:=ParGlbwMod[6]; end;


function S:Real;  begin S:=1/(z2-z1); end;
function A1:Real; begin A1:=MaxR(1E-12,y1-g1); end;
function X1:Real; begin X1:=(1-y1)*z1+y1*z2; end;
function X2:Real; begin X2:=(1-y2)*z1+y2*z2; end;
function A2:Real; begin A2:=MinR(-1E-12,y2-g2); end;
function B1:Real; begin B1:=(1-g1)*z1+g1*z2; end;
function B2:Real; begin B2:=(1-g2)*z1+g2*z2; end;


function U1(x:Real):Real;
 begin U1:=Exp(MaxR(-70,MinR(70,-2*S/A1*(x-X1)))); end;

function U2(x:Real):Real;
 begin U2:=Exp(MaxR(-70,MinR(70,-2*S/A2*(x-X2)))); end;

function W1(x:Real):Real;
 begin W1:=4*U1(x)/SQR(1+U1(x)); end;

function W2(x:Real):Real;
 begin W2:=4*U2(x)/SQR(1+U2(x)); end;

function Yl(x:Real):Real;
 begin Yl:=g1+2*A1/(1+U1(x)); end;

function Ym(x:Real):Real;
 begin Ym:=S*(x-z1); end;

function Yr(x:Real):Real;
 begin Yr:=g2+2*A2/(1+U2(x)); end;

function GlbwMod(x:Real):Real;
 begin
 if       x<x1 then GlbwMod:=Yl(x)
 else if x<=x2 then GlbwMod:=Ym(x)
 else               GlbwMod:=Yr(x);
 end;


function Dylnachz1(x:Real):Real;
 begin Dylnachz1:=SQR(S)*W1(x)*(x-z2); end;

function Dylnachg1(x:Real):Real;
 begin Dylnachg1:=1-2/(1+U1(x))+S/A1*W1(x)*(x-X1); end;

function Dylnachy1(x:Real):Real;
 begin Dylnachy1:=2/(1+U1(x))-S/A1*W1(x)*(x-B1); end;

function Dylnachy2(x:Real):Real;
 begin Dylnachy2:=0; end;

function Dylnachg2(x:Real):Real;
 begin Dylnachg2:=0; end;

function Dylnachz2(x:Real):Real;
 begin Dylnachz2:=-SQR(S)*W1(x)*(x-z1); end;


function Dymnachz1(x:Real):Real;
 begin Dymnachz1:=SQR(S)*(x-z2); end;

function Dymnachg1(x:Real):Real;
 begin Dymnachg1:=0; end;

function Dymnachy1(x:Real):Real;
 begin Dymnachy1:=0; end;

function Dymnachy2(x:Real):Real;
 begin Dymnachy2:=0; end;

function Dymnachg2(x:Real):Real;
 begin Dymnachg2:=0; end;

function Dymnachz2(x:Real):Real;
 begin Dymnachz2:=-SQR(S)*(x-z1); end;


function Dyrnachz1(x:Real):Real;
 begin Dyrnachz1:=SQR(S)*W2(x)*(x-z2); end;

function Dyrnachg1(x:Real):Real;
 begin Dyrnachg1:=0; end;

function Dyrnachy1(x:Real):Real;
 begin Dyrnachy1:=0; end;

function Dyrnachy2(x:Real):Real;
 begin Dyrnachy2:=2/(1+U2(x))-S/A2*W2(x)*(x-B2); end;

function Dyrnachg2(x:Real):Real;
 begin Dyrnachg2:=1-2/(1+U2(x))+S/A2*W2(x)*(x-X2); end;

function Dyrnachz2(x:Real):Real;
 begin Dyrnachz2:=-SQR(S)*W2(x)*(x-z1); end;


function DGlbwnachz1(x:Real):Real;
 begin
 if       x<x1 then DGlbwnachz1:=Dylnachz1(x)
 else if x<=x2 then DGlbwnachz1:=Dymnachz1(x)
 else               DGlbwnachz1:=Dyrnachz1(x);
 end;

function DGlbwnachg1(x:Real):Real;
 begin
 if       x<x1 then DGlbwnachg1:=Dylnachg1(x)
 else if x<=x2 then DGlbwnachg1:=Dymnachg1(x)
 else               DGlbwnachg1:=Dyrnachg1(x);
 end;

function DGlbwnachy1(x:Real):Real;
 begin
 if       x<x1 then DGlbwnachy1:=Dylnachy1(x)
 else if x<=x2 then DGlbwnachy1:=Dymnachy1(x)
 else               DGlbwnachy1:=Dyrnachy1(x);
 end;

function DGlbwnachy2(x:Real):Real;
 begin
 if       x<x1 then DGlbwnachy2:=Dylnachy2(x)
 else if x<=x2 then DGlbwnachy2:=Dymnachy2(x)
 else               DGlbwnachy2:=Dyrnachy2(x);
 end;

function DGlbwnachg2(x:Real):Real;
 begin
 if       x<x1 then DGlbwnachg2:=Dylnachg2(x)
 else if x<=x2 then DGlbwnachg2:=Dymnachg2(x)
 else               DGlbwnachg2:=Dyrnachg2(x);
 end;

function DGlbwnachz2(x:Real):Real;
 begin
 if       x<x1 then DGlbwnachz2:=Dylnachz2(x)
 else if x<=x2 then DGlbwnachz2:=Dymnachz2(x)
 else               DGlbwnachz2:=Dyrnachz2(x);
 end;


procedure NablaGlbw(x:Real; Var GradvektGlbw:ParGlbwMod_T);
 begin
 GradvektGlbw[1]:=DGlbwnachz1(x);
 GradvektGlbw[2]:=DGlbwnachg1(x);
 GradvektGlbw[3]:=DGlbwnachy1(x);
 GradvektGlbw[4]:=DGlbwnachy2(x);
 GradvektGlbw[5]:=DGlbwnachg2(x);
 GradvektGlbw[6]:=DGlbwnachz2(x);
 end;


begin
end.