unit JS_ZEPAS;


Interface

Uses
 crt, dos, js_sub, matrix;

Const
 MxCls=10;                           {Maximale Anzahl Klassen}

Type
 Strt_T   = (strtno,strtms,strtlu);  {StrategieTyp}
 ClsTab_T = array[1..MxCls] of Char; {Liste der Klassen}
 RdbTyp_T = (apoprb,clcprb);         {Typ der RadBas-Approximation}

 StiDsc_T = record
             DscDatei: Pathstr;      {Pfadname der Beschreibungsdatei}
             StiDatei: PathStr;      {Pfadname der Stichprobendatei}
             LengthDs: Word;         {Lnge des Datensatzes}
             PosLabel: Word;         {Pos(Label) im Datensatz, 0 -> es gibt keinen}
             PosIdent: Word;         {Pos(Ident) im Datensatz, 0 -> es gibt keinen}
             UseVon:   Word;         {Benutze Element von ... im Datensatz}
             UseBis:   Word;         {Benutze Element bis ... im Datensatz}
             LengthMv: Word;         {Lnge des Merkmalsvektors}
             Classes:  Set of Char;  {darf nicht leer sein}
             SmplPCls: Word;         {pro Label, 0 -> alle Merkmalsvektoren}
            end;

 PslDsc_T = record
             DscDatei: PathStr;     {Pfadname der Beschreibungsdatei}
             ErzPsl:   array[0..4] of Word; {Erzeugungsvorschrift PSL}
             LengthMv: Word;        {Lnge des Merkmalsvektors}
             LengthPv: Word;        {resultierende Lnge des Polynomvektors}
            end;

 MomDsc_T = record
             DscDatei: Pathstr;     {Pfadname der Beschreibungsdatei}
             MomDatei: PathStr;     {Pfadname der Momentenmatrixdateien, Namenskopf}
            end;

 RgrDsc_T = record
             DscDatei: PathStr;     {Pfadname der Beschreibungsdatei}
             PrtDatei: PathStr;     {Pfadname der Protokolldatei}
             KlfDatei: PathStr;     {Pfadname der Klassifikatordateien, Namenskopf}
             AnzX:     Word;        {Anzahl Einflugren}
             AnzY:     Word;        {Anzahl Zielgren}
             Strt:     Strt_T;      {Strategie: No/Lu/Ms}
             X0First:  Boolean;     {konstanten Term X0=1 vorziehen}
             MinDiag:  Extended;    {MinDiag-Schwelle}
             StepPrt:  Word;        {Schrittweite fr das Protokoll}
             StepKlf:  Word;        {Schrittweite fr die Klassifikatorausgabe}
            end;

 Rezept_T = record
             RezeptDatei:   PathStr;         {Pfadname der Rezeptdatei}
             NumbOfContrib: Word;                         {Anzahl Beitrge}
             ClsLabel:      array[1..MxCls] of Char;      {Label}
             Share:         array[1..MxCls] of Extended;  {Anteil}
             MDateiName:    array[1..MxCls] of PathStr;   {Momentenmatrix}
             BelongsTo:     array[1..MxCls] of Word;      {Hinweis auf Klasse}
             NumbOfClasses: Word;                         {Anzahl Klassen}
             Classes:       ClsTab_T;                     {Liste der Klassen}
            end;


 PslDat_T = record
             LengthMv: Word;        {Lnge des Merkmalsvektors}
             LengthPv: Word;        {resultierende Lnge des Polynomvektors}
             Psl: array[0..256,1..4] of Word;          {Psl in Tabellenform}
            end;


 RdbDat_P = ^RdbDat_T;
 RdbDat_T = object
             LengthMv: Word;        {Lnge des Merkmalsvektors}
             LengthPv: Word;        {resultierende Lnge des Radbasvektors}
             cen:      array[1..20] of PMatrix;  {Radbas-Cemter}
             kovi:     array[1..20] of PMatrix;  {Radbas-Inverse Kovars}
             sqrtdeti: array[1..20] of Extended; {RadBas-Determ. invers Sqrt}

             constructor Init(_lx:Word);
             constructor InitLoad(datei:PathStr; fak:Real);
             destructor  Done;

             function Print:word;
            end;


 MomDat_P = ^MomDat_T;
 MomDat_T = object
             Matrix: PMatrix;      {MomentenMatrix}
             PslDat: PslDat_T;     {Psl}
             class : char;         {Label}
             count : word;         {Anzahl Sample}

             constructor Init(_name:string; _row,_col:word; _psldat:PslDat_T; _class:char; _count:word);
             constructor InitLoad(s:string);
             constructor InitCopy(m:MomDat_P);
             destructor  Done;

             function Save(s:string):word; virtual;
             function Load(s:string):word; virtual;

             function AddGew(m:MomDat_P;gewicht:extended):word;
             function PrintMatrix(nachkomma:word):word;
             function PrintPsl:word;

             function WriteMatrix(s:PathStr; l,n:word):word;
            end;


 ClfDat_P = ^ClfDat_T;
 ClfDat_T = object
             Matrix:        PMatrix;      {KlassifikatorMatrix}
             PslDat:        PslDat_T;     {Psl}
             NumbOfClasses: Word;         {Anzahl Klassen}
             Classes:       ClsTab_T;     {Liste der Klassen}

             constructor Init(_name:string; _row,_col:word; _psldat:PslDat_T; _ncls:Word; _cls:ClsTab_T);
             constructor InitLoad(s:string);
             constructor InitCopy(k:ClfDat_P);
             destructor  Done;

             function Save(s:string):word; virtual;
             function Load(s:string):word; virtual;

             function PrintMatrix(nachkomma:word):word;
             function PrintPsl:word;
            end;

Const
 ErrorCodeStr: array[1..7] of string= ('Kann Datei nicht finden',
                                       'Kann Datei nicht zum Schreiben ffnen',
                                       'Momentenmatrix hat falsches Format',
                                       'Psln stimmen nicht berein',
                                       '',
                                       '',
                                       '');


Function GetStiDsc(Var StiDsc:StiDsc_T):Word;
Function GetPslDsc(Var PslDsc:PslDsc_T):Word;
Function GetMomDsc(Var MomDsc:MomDsc_T):Word;
Function GetRezept(Var Rezept:Rezept_T):Word;
Function GetRgrDsc(Var RgrDsc:RgrDsc_T):Word;

Function GetRdbKlf(d:PathStr; Var ClfDat:ClfDat_P):Word;

Function BinomKoeff(n,g:Word):Word;

Function LeerPsl(Var Psldat:PslDat_T):Word;
Function LeerClsLst(Var ClsLst:ClsTab_T):Word;
Function MkPsl(PslDsc:PslDsc_T; Var PslDat:PslDat_T):Word;
Function VToZ(PslDat:PslDat_T; v:PMatrix; Var z:PMatrix):Word;
Function VToX(PslDat:PslDat_T; v:PMatrix; Var x:PMatrix):Word;
Function AkkuMom(z:PMatrix; Var mom:MomDat_P):Word;
Function CmplMom(Var mom:MomDat_P):Word;
Function RunMomMat(StiDsc:StiDsc_T; PslDat:PslDat_T; MomDsc:MomDsc_T):Word;
Function RunRdbMomMat(StiDsc:StiDsc_T; RdbDat:RdbDat_P; RdbTyp:RdbTyp_T; PslDat:PslDat_T; MomDsc:MomDsc_T):Word;

Function Rgr(psldat:PslDat_T; classes:ClsTab_T; RgrDsc:RgrDsc_T; Var Mat:PMatrix):Word;
Function RunMkQMK(Rezept:Rezept_T; RgrDsc:RgrDsc_T):Word;

Function RunClsf(StiDsc:StiDsc_T; klfdatei,dvektdatei:PathStr):Word;
Function RunRdbClsf(StiDsc:StiDsc_T; RdbDat:RdbDat_P; RdbTyp:RdbTyp_T; klfdatei,dvektdatei:PathStr):Word;

function RadBasVtoX(RdbDat:RdbDat_P; RdbTyp:RdBtyp_T; v:PMatrix; Var x:PMatrix):Word;
function RunRdb(StiDsc:StiDsc_T; RdbDat:RdbDat_P; RdbTyp:RdbTyp_T; ErgStiDatei:PathStr):Word;


Implementation

(*********************** MomDat_T *********************************)

constructor MomDat_T.Init(_name:string; _row,_col:word; _psldat:PslDat_T; _class:char; _count:word);
 Var
  i,j:Word;
 begin
 Matrix:=New(PMatrix,Init(_name,_row,_col));
 for i:=0 to Matrix^.row-1 do
  for j:=0 to Matrix^.col-1 do
   Matrix^.WriteMatrixElement(i,j,0);
 PslDat:=_psldat;
 Count:=_count;
 Class:=_class;
 end;

constructor MomDat_T.InitLoad(s:string);
 begin
 LeerPsl(psldat);
 Init('_INIT',1,1,psldat,' ',0);
 Load(s);
 end;

constructor MomDat_T.InitCopy(m:MomDat_P);
 var
  n:MomDat_P;
 begin
 Init(m^.Matrix^.name,m^.Matrix^.row,m^.Matrix^.col,m^.psldat,m^.class,m^.count);
 Matrix^.FillWith(m^.Matrix);
 end;

destructor MomDat_T.Done;
 begin
 Dispose(Matrix,Done);
 end;

function MomDat_T.Save(s:string):word;
 var
  f:file;
 begin
 assign(f,s);
 rewrite(f,1);
 blockwrite(f,Matrix^.name,33);
 blockwrite(f,Matrix^.row,2);
 blockwrite(f,Matrix^.col,2);
 blockwrite(f,Matrix^.data^,Matrix^.row*Matrix^.col*10);
 blockwrite(f,PslDat,2+257*4*2);
 blockwrite(f,class,1);
 blockwrite(f,count,2);
 close(f);
 end;

function MomDat_T.Load(s:string):word;
 var
  f:file;
 begin
 assign(f,s);
 reset(f,1);
 blockread(f,Matrix^.name,33);
 FreeMem(Matrix^.data,Matrix^.row*Matrix^.col*10);
 blockread(f,Matrix^.row,2);
 blockread(f,Matrix^.col,2);
 GetMem(Matrix^.data,Matrix^.row*Matrix^.col*10);
 blockread(f,Matrix^.data^,Matrix^.row*Matrix^.col*10);
 blockread(f,PslDat,2+257*4*2);
 blockread(f,class,SizeOf(class));
 blockread(f,count,SizeOf(count));
 close(f);
 end;


function MomDat_T.AddGew(m:MomDat_P;gewicht:extended):word;
var temp:MomDat_P;
 begin
 temp:=New(MomDat_P,InitCopy(m));
 Matrix^.MultWithFactor(1-gewicht);
 temp^.Matrix^.MultWithFactor(gewicht);
 Matrix^.Add(temp^.Matrix);
 Dispose(temp,done);
 end;

function MomDat_T.PrintMatrix(nachkomma:word):word;
 begin
 writeln;
 write('Class: ',class,'    Count: ',count);
 Matrix^.Print(nachkomma);
 end;


function MomDat_T.WriteMatrix(s:PathStr; l,n:word):word;
 Var
  f:Text; i,j:Word; w:Extended;
 begin
 assign(f,s); rewrite(f);
 writeln(f,'Momentenmatrix');
 writeln(f,'Klasse: ',class,'    Anzahl: ',count);
 writeln(f);
 writeln(f,'Zeilen: ',Matrix^.Row,'    Spalten: ',Matrix^.Col);
 writeln(f);
 for i:=0 to Matrix^.Row-1 do
  begin
  for j:=0 to Matrix^.Col-1 do
   begin
   Matrix^.ReadMatrixElement(i,j,w);
   write(f,w:l:n);
   end;
  writeln(f);
  end;
 Close(f);
 end;


function MomDat_T.PrintPsl:Word;
 Var
  p,i:Word;
 begin
 writeln('Length v: ',PslDat.LengthMv,' -> Length x: ',PslDat.LengthPv);
 writeln;
 writeln('   #   1   2   3   4');
 writeln('--------------------');
 for p:=0 to PslDat.LengthPv-1 do
  begin
  write(p:4);
  for i:=1 to 4 do write(PslDat.Psl[p,i]:4);
  writeln;
  end;
 end;



(*********************** ClfDat_T *********************************)

constructor ClfDat_T.Init(_name:string; _row,_col:word; _psldat:PslDat_T; _ncls:Word; _cls:ClsTab_T);
begin
 Matrix:=New(PMatrix,Init(_name,_row,_col));
 PslDat:=_psldat;
 NumbOfClasses:=_ncls;
 Classes:=_cls;
end;

constructor ClfDat_T.InitLoad(s:string);
begin
 LeerPsl(PslDat);
 LeerClsLst(Classes);
 Init('_INIT',1,1,psldat,0,Classes);
 Load(s);
end;

constructor ClfDat_T.InitCopy(k:ClfDat_P);
var n:ClfDat_P;
begin
 Init(k^.Matrix^.name,k^.Matrix^.row,k^.Matrix^.col,k^.PslDat,k^.NumbOfClasses,k^.Classes);
 Matrix^.FillWith(k^.Matrix);
end;

destructor ClfDat_T.Done;
begin
 Dispose(Matrix,Done);
end;

function ClfDat_T.Save(s:string):word;
var f:file;
begin
 assign(f,s);
 rewrite(f,1);
 blockwrite(f,Matrix^.name,33);
 blockwrite(f,Matrix^.row,2);
 blockwrite(f,Matrix^.col,2);
 blockwrite(f,Matrix^.data^,Matrix^.row*Matrix^.col*10);
 blockwrite(f,PslDat,2+257*4*2);
 blockwrite(f,NumbOfClasses,2);
 blockwrite(f,Classes,MxCls);
 close(f);
end;

function ClfDat_T.Load(s:string):word;
var f:file;
begin
 assign(f,s);
 reset(f,1);
 blockread(f,Matrix^.name,33);
 FreeMem(Matrix^.data,Matrix^.row*Matrix^.col*10);
 blockread(f,Matrix^.row,2);
 blockread(f,Matrix^.col,2);
 GetMem(Matrix^.data,Matrix^.row*Matrix^.col*10);
 blockread(f,Matrix^.data^,Matrix^.row*Matrix^.col*10);
 blockread(f,PslDat,2+257*4*2);
 blockread(f,NumbOfClasses,2);
 blockread(f,Classes,MxCls);
 close(f);
end;

function ClfDat_T.PrintMatrix(nachkomma:word):word;
 Var k:Word;
begin
 writeln;
 writeln('NumbOfClasses: ',NumbOfClasses);
 for k:=1 to NumbOfClasses do write(Classes[k],' ');
 Matrix^.Print(nachkomma);
end;

function ClfDat_T.PrintPsl:Word;
 Var
  p,i:Word;
 begin
 writeln('Length v: ',PslDat.LengthMv,' -> Length x: ',PslDat.LengthPv);
 writeln;
 writeln('   #   1   2   3   4');
 writeln('--------------------');
 for p:=0 to PslDat.LengthPv-1 do
  begin
  write(p:4);
  for i:=1 to 4 do write(PslDat.Psl[p,i]:4);
  writeln;
  end;
 end;


(*********************** RdbDat_T *********************************)

constructor RdbDat_T.Init(_lx:Word);
 Var
  i: Word;
 begin
  LengthMv:=2;
  LengthPv:=_lx;
  for i:=1 to LengthPv do
   begin
   cen[i]:= New(PMatrix,Init('CEN'+Trim(Stri(i,2)),LengthMv,1));
   kovi[i]:=New(PMatrix,Init('KOV'+Trim(Stri(i,2)),LengthMv,LengthMv));
   sqrtdeti[i]:=0;
   end;
 end;


constructor RdbDat_T.InitLoad(datei:PathStr; fak:Real);
 Var
  f:       Text;
  zeile,s: String;
  i:       Word;
  w:       Extended;

 begin
 Assign(f,datei); {$I-} Reset(f); {$I+}
 if IOResult=0 then

  begin
  i:=0;
  while not EOF(f) do
   begin
   readln(f,zeile); GetPar(zeile); s:=ParStr(1);
   if (ParCount=7) and (s[1]<>'*') then Inc(i);
   end;

  Init(i);

  Reset(f); i:=0;
  while not EOF(f) do
   begin
   readln(f,zeile); GetPar(zeile); s:=ParStr(1);
   if (ParCount=7) and (s[1]<>'*') then
    begin
    Inc(i);
    cen[i]^.WriteMatrixElement(0,0,ValR(ParStr(1)));
    cen[i]^.WriteMatrixElement(1,0,ValR(ParStr(2)));
    kovi[i]^.WriteMatrixElement(0,0,sqr(fak)*ValR(ParStr(3)));
    kovi[i]^.WriteMatrixElement(0,1,sqr(fak)*ValR(ParStr(4)));
    kovi[i]^.WriteMatrixElement(1,0,sqr(fak)*ValR(ParStr(5)));
    kovi[i]^.WriteMatrixElement(1,1,sqr(fak)*ValR(ParStr(6)));
    kovi[i]^.Invert;
    Matrix_Det(kovi[i],w);
    sqrtdeti[i]:=Sqrt(w);
    end;
   end;
  Close(f);
  end;
 end;


destructor RdbDat_T.Done;
 Var
  i:Word;
 begin
 for i:=1 to LengthPv do
  begin
  Dispose(cen[i], Done);
  Dispose(kovi[i],Done);
  end;
 end;


function RdbDat_T.Print:Word;
 Var
  i,j: Word;
 begin
 writeln('Length v: ',LengthMv,' -> Length x: ',LengthPv);
 for i:=1 to LengthPv do
  begin
  cen[i]^.print(4);
  kovi[i]^.print(4);
  writeln('SqrtDetInv: ',sqrtdeti[i]:12:4);
  end;
 end;



(*****************************************************************)




Function BinomKoeff(n,g:Word):Word;
 Var Zz,Nn,Gg:LongInt;
 begin
 Zz:=1; Nn:=1;
 For gg:=g downto 1 do begin Zz:=Zz*(n+1-gg); Nn:=Nn*gg; end;
  BinomKoeff:=Zz div Nn;
 end;



Function GetStiDsc(Var StiDsc:StiDsc_T):Word;
 Const
  AnzKeyWord=5;
  KeyWord: array[1..AnzKeyword] of string = ('FORMAT:',
                                             'USE:',
                                             'CLASSES:',
                                             'SAMPLEPERCLASS:',
                                             'FILE:');
 Var
  f:       Text;
  zeile,s: string;
  i,j,k:   Word;

 begin
  GetStiDsc:=0;
  with StiDsc do
   begin
   Assign(f,DscDatei); {$I-} Reset(f); {$I+}
   if not IOResult=0 then GetStiDsc:=1 else
    begin
    while not EOF(f) do
     begin
     readln(f,zeile); GetPar(zeile);
     if (ParCount>=1) then
      begin
      i:=1;
      while (i<=AnzKeyWord) and (KeyWord[i]<>UpCaseWord(ParStr(1))) do Inc(i);
      case i of
       1: {Format}
          begin
          LengthDs:=0; PosLabel:=0; PosIdent:=0;
          for j:=2 to ParCount do
           if      ParStr(j)='Label' then begin Inc(LengthDs); Poslabel:=LengthDs; end
           else if ParStr(j)='Identifier' then begin Inc(LengthDs); PosIdent:=LengthDs; end
           else LengthDs:=LengthDs+ValI(ParStr(j));
          end;
       2: {Use}
          begin
          UseVon:=ValI(ParStr(2));
          UseBis:=ValI(ParStr(4));
          LengthMv:=UseBis-UseVon+1;
          end;
       3: {Select}
          begin
          Classes:=[];
          for j:=2 to ParCount do
           begin
           s:=ParStr(j);
           for k:=1 to Length(s) do Classes:=Classes+[s[k]];
           end;
          end;
       4: {SamplePerClass}
          begin
          SmplPCls:=0;
          if ParCount=2 then SmplPCls:=ValI(ParStr(2));
          end;
       5: {File}
          begin
          StiDatei:=ParStr(2);
          end;
      end;
      end;
     end;
    end;
   Close(f);
   end;
 end;



Function GetPslDsc(Var PslDsc:PslDsc_T):Word;
 Const
  AnzKeyWord=5;
  KeyWord: array[1..AnzKeyword] of string = ('CONSTANT:',
                                             'LINEAR:',
                                             'QUADRATIC:',
                                             'CUBIC:',
                                             'FOURTHDGR:');
 Var
  f:     Text;
  zeile: string;
  i:     Word;

 begin
  GetPslDsc:=0;

  with PslDsc do
   begin
   LengthMv:=0;
   For i:=0 to 4 do ErzPsl[i]:=0;
   Assign(f,DscDatei); {$I-} Reset(f); {$I+}
   if not IOResult=0 then GetPslDsc:=1 else
    begin
    while not EOF(f) do
     begin
     readln(f,zeile); GetPar(zeile);
     if (ParCount>=1) then
      begin
      i:=1;
      while (i<=AnzKeyWord) and (KeyWord[i]<>UpCaseWord(ParStr(1))) do Inc(i);
      case i of
       1: {Constant}
          ErzPsl[0]:=ValI(ParStr(2));
       2: {Linear}
          ErzPsl[1]:=ValI(ParStr(2));
       3: {Quadratic}
          ErzPsl[2]:=ValI(ParStr(2));
       4: {Cubic}
          ErzPsl[3]:=ValI(ParStr(2));
       5: {FourthDgr}
          ErzPsl[4]:=ValI(ParStr(2));
      end;
      end;
     end;
    end;
   Close(f);
   if ErzPsl[0]=0 then LengthPv:=0 else LengthPv:=1;
   for i:=1 to 4 do LengthPv:=LengthPv+BinomKoeff(ErzPsl[i]+i-1,i);
   end;
 end;


Function GetMomDsc(Var MomDsc:MomDsc_T):Word;
 Const
  AnzKeyWord=1;
  KeyWord: array[1..AnzKeyword] of string = ('FILE:');

 Var
  f:     Text;
  zeile: string;
  i:     Word;

 begin
  GetMomDsc:=0;
  with MomDsc do
   begin
   Assign(f,DscDatei); {$I-} Reset(f); {$I+}
   if not IOResult=0 then GetMomDsc:=1 else
    begin
    while not EOF(f) do
     begin
     readln(f,zeile); GetPar(zeile);
     if (ParCount>=1) then
      begin
      i:=1;
      while (i<=AnzKeyWord) and (KeyWord[i]<>UpCaseWord(ParStr(1))) do Inc(i);
      case i of
       1: {File}
          MomDatei:=ParStr(2);
      end;
      end;
     end;
    end;
   Close(f);
   end;
 end;


Function GetRgrDsc(Var RgrDsc:RgrDsc_T):Word;
 Const
  AnzKeyWord=7;
  KeyWord: array[1..AnzKeyword] of string = ('STRATEGIE:',
                                             'X0FIRST:',
                                             'MINDIAG:',
                                             'STEPPROTOCOL:',
                                             'STEPCLASSIFIER:',
                                             'PROTFILE:',
                                             'CLSFFILE:');
 Var
  f:       Text;
  zeile,s: string;
  i,j,k:   Word;

 begin
  GetRgrDsc:=0;
  with RgrDsc do
   begin
   Assign(f,DscDatei); {$I-} Reset(f); {$I+}
   if not IOResult=0 then GetRgrDsc:=1 else
    begin
    while not EOF(f) do
     begin
     readln(f,zeile); GetPar(zeile);
     if (ParCount>=1) then
      begin
      i:=1;
      while (i<=AnzKeyWord) and (KeyWord[i]<>UpCaseWord(ParStr(1))) do Inc(i);
      case i of
       1: {Strategie}
          if UpCaseWord(ParStr(2))='LU' then strt:=strtlu else
          if UpCaseWord(ParStr(2))='MS' then strt:=strtms
          else strt:=strtno;
       2: {X0First}
          X0First:=(UpCaseword(ParStr(2))='TRUE');
       3: {MinDiag}
          MinDiag:=ValR(ParStr(2));
       4: {StepProtocol}
          StepPrt:=ValI(ParStr(2));
       5: {StepClassifier}
          StepKlf:=ValI(ParStr(2));
       6: {ProtFile}
          PrtDatei:=ParStr(2);
       7: {ProtFile}
          KlfDatei:=ParStr(2);
      end;
      end;
     end;
    end;
   Close(f);
   end;
 end;


Function GetRdbKlf(d:PathStr; Var ClfDat:ClfDat_P):Word;
 Const
  AnzKeyWord=5;
  KeyWord: array[1..AnzKeyword] of string = ('NUMBOFCLASSES:',
                                             'NUMBOFRDB:',
                                             'CLASSES:',
                                             'APRIO:',
                                             'KOEFF:');
 Var
  f:         Text;
  zeile,s:   string;
  i,j,k:     Word;
  NCls,NRdb: Word;
  ClsTab:    ClsTab_T;
  PslDsc:    PslDsc_T;
  PslDat:    PslDat_T;
  APrio:     Array[1..MxCls] of Real;
  sum:       Real;

 begin
  GetRdbKlf:=0;
  Assign(f,d); {$I-} Reset(f); {$I+}
  if not IOResult=0 then GetRdbKlf:=1 else
   begin
   while not EOF(f) do
    begin
    readln(f,zeile); GetPar(zeile);
    if (ParCount>=1) then
     begin
     i:=1;
     while (i<=AnzKeyWord) and (KeyWord[i]<>UpCaseWord(ParStr(1))) do Inc(i);
     case i of
      1: {NumbOfClasses}
         NCls:=Vali(ParStr(2));
      2: {NumbOfRdb}
         NRdb:=Vali(ParStr(2));
      3: {Classes}
         for k:=1 to NCls do begin s:=ParStr(k+1); ClsTab[k]:=s[1]; end;
      4: {Aprio}
         begin
         sum:=0;
         for k:=1 to NCls do begin
                             APrio[k]:=ValR(ParStr(k+1));
                             sum:=sum+APrio[k];
                             end;
         for k:=1 to NCls do APrio[k]:=APrio[k]/sum;
         end;
      5: {Koeff}
         begin
         PslDsc.DscDatei:='';
         PslDsc.ErzPsl[1]:=NRdb;
         PslDsc.ErzPsl[2]:=0;
         PslDsc.ErzPsl[3]:=0;
         PslDsc.ErzPsl[4]:=0;
         PslDsc.LengthMv:=NRdb;
         PslDsc.LengthPv:=NRdb;
         MkPsl(PslDsc,PslDat);
         ClfDat:=New(ClfDat_P,Init('Klf',NRdb,Ncls,PslDat,NCls,ClsTab));
         j:=0;
         for k:=1 to NCls do
          ClfDat^.Matrix^.WriteMatrixElement(0,k-1,APrio[k]*ValR(ParStr(k+1)));
         end;
      6: if ParCount=Ncls then
         begin
         Inc(j);
         for k:=1 to NCls do
          ClfDat^.Matrix^.WriteMatrixElement(j,k-1,APrio[k]*ValR(ParStr(k)));
         end;
     end;
     end;
    end;
   Close(f);
   end;
 end;




Function LeerPsl(Var Psldat:PslDat_T):Word;
 Var
  p,i:Word;
 begin
 LeerPsl:=0;
 PslDat.LengthMv:=0;
 PslDat.LengthPv:=0;
 for p:=0 to 256 do
  for i:=1 to 4 do PslDat.Psl[p,i]:=0;
 end;


Function LeerClsLst(Var ClsLst:ClsTab_T):Word;
 Var
  k:Word;
 begin
 LeerClsLst:=0;
 for k:=1 to MxCls do ClsLst[k]:=' ';
 end;


Function MkPsl(PslDsc:PslDsc_T; Var PslDat:PslDat_T):Word;
 Var
  p,i,j,k,l,m:  Integer;
  w1,w2,w3,w4:  Extended;
 begin
 with PslDsc do
  begin
  MkPsl:=0;
  PslDat.LengthMv:=LengthMv;
  PslDat.LengthPv:=LengthPv;
  p:=0;
  {************************************* konstanter Term}
  if ErzPsl[0]<>0 then
   begin PslDat.Psl[p,1]:=0; Inc(p); end;
  {*************************************** lineare Terme}
  m:=MinI(LengthMv,ErzPsl[1]);
  for i:=1 to m do
   begin
   PslDat.Psl[p,1]:=i;
   Inc(p);
   end;
  {********************************** quadratische Terme}
  m:=MinI(LengthMv,ErzPsl[2]);
  for i:=1 to m do
   for j:=i to m do
    begin
    PslDat.Psl[p,1]:=i;
    PslDat.Psl[p,2]:=j;
    Inc(p);
    end;
  {************************************** kubische Terme}
  m:=MinI(LengthMv,ErzPsl[3]);
  for i:=1 to m do
   for j:=i to m do
    for k:=j to m do
     begin
     PslDat.Psl[p,1]:=i;
     PslDat.Psl[p,2]:=j;
     PslDat.Psl[p,3]:=k;
     Inc(p);
     end;
  {******************************** Terme vierten Grades}
  m:=MinI(LengthMv,ErzPsl[4]);
  for i:=1 to m do
   for j:=i to m do
    for k:=j to m do
     for l:=k to m do
      begin
      PslDat.Psl[p,1]:=i;
      PslDat.Psl[p,2]:=j;
      PslDat.Psl[p,3]:=k;
      PslDat.Psl[p,4]:=l;
      Inc(p);
      end;
  end;
 end;


Function VToZ(PslDat:PslDat_T; v:PMatrix; Var z:PMatrix):Word;
 Var
  p,i:   Integer;
  w,w1:  Extended;

 begin
 with PslDat do
  begin
  VtoZ:=0;
  for p:=0 to LengthPv-1 do
   begin
   w:=1; i:=1;
   while (i<=4) and (Psl[p,i]>0) do
    begin
    v^.ReadMatrixElement(Psl[p,i]-1,0,w1);
    w:=w*w1;
    Inc(i);
    end;
   z^.WriteMatrixElement(p,0,w);
   end;
  {**************************************** Klassenteil}
  z^.WriteMatrixElement(LengthPv,0,1);
  end;
 end;


Function VToX(PslDat:PslDat_T; v:PMatrix; Var x:PMatrix):Word;
 Var
  p,i:   Integer;
  w,w1:  Extended;

 begin
 with PslDat do
  begin
  VtoX:=0;
  for p:=0 to LengthPv-1 do
   begin
   w:=1; i:=1;
   while (i<=4) and (Psl[p,i]>0) do
    begin
    v^.ReadMatrixElement(Psl[p,i]-1,0,w1);
    w:=w*w1;
    Inc(i);
    end;
   x^.WriteMatrixElement(p,0,w);
   end;
  end;
 end;




Function AkkuMom(z:PMatrix; Var mom:MomDat_P):Word;
 Var
  i,j:     Word;
  w,w1,w2: Extended;
 begin
 for i:=0 to mom^.Matrix^.row-1 do
  for j:=i to mom^.Matrix^.col-1 do
   begin
   z^.ReadMatrixElement(i,0,w1);
   z^.ReadMatrixElement(j,0,w2);
   mom^.Matrix^.ReadMatrixElement(i,j,w);
   mom^.Matrix^.WriteMatrixElement(i,j,w+w1*w2);
   end;
 end;



Function CmplMom(Var mom:MomDat_P):Word;
 Var
  i,j:  Integer;
  w,m:  Extended;
 begin
 CmplMom:=0;
 mom^.Matrix^.ReadMatrixElement(mom^.Matrix^.row-1,mom^.Matrix^.col-1,m);
 for i:=0 to mom^.Matrix^.row-1 do
  begin
  for j:=0 to i-1 do
   begin
   mom^.Matrix^.ReadMatrixElement(j,i,w);
   mom^.Matrix^.WriteMatrixElement(i,j,w);
   end;
  for j:=i to mom^.Matrix^.col-1 do
   begin
   mom^.Matrix^.ReadMatrixElement(i,j,w);
   mom^.Matrix^.WriteMatrixElement(i,j,w/m);
   end;
  end;
 end;


Function RunMomMat(StiDsc:StiDsc_T; PslDat:PslDat_T; MomDsc:MomDsc_T):Word;

 Var
  f:          Text;
  mdateiname: PathStr;
  zeile:      String;
  SmplCls:    Char;
  Fertig:     Boolean;
  ClsCount:   Word;
  i,j:        Word;
  Cls:        Char;
  v,z:        PMatrix;
  MomDat:     MomDat_P;

 begin
 RunMomMat:=0;
 v:=New(PMatrix,Init('V',StiDsc.LengthMv,1));
 z:=New(PMatrix,Init('Z',PslDat.LengthPv+1,1));
 MomDat:=New(MomDat_P,Init('Mom',PslDat.LengthPv+1,PslDat.LengthPv+1,PslDat,' ',0));

 with StiDsc do
   begin
   Assign(f,StiDatei); {$I-} Reset(f); {$I+}
   if not (IOResult=0) then RunMomMat:=1 else
    begin
    for i:=32 to 255 do
     begin
     Cls:=Char(i);
     if Cls in Classes then

      {************************************** Bearbeiten einer Klasse}
      begin
      Reset(f); Fertig:=False; ClsCount:=0; MomDat^.Matrix^.Initialize;
      while not Fertig do
       begin
       readln(f,zeile); GetPar(zeile);
       if ParStr(PosLabel)=Cls then
          begin
          Inc(ClsCount);
          Fertig:= EOF(f) or (ClsCount>=StiDsc.SmplPCls);
          for j:=0 to LengthMv-1 do v^.WriteMatrixElement(j,0,ValR(ParStr(UseVon+j)));

          {************************************** Verarbeitung Sample}
          VtoZ(PslDat,v,z);
          AkkuMom(z,MomDat);
          {********************************* Ende Verarbeitung Sample}

          end;
       end;

       CmplMom(MomDat);
       mdateiname:=MomDsc.MomDatei+Cls+'.zmm';

       MomDat^.Class:=Cls;
       MomDat^.Count:=ClsCount;
       MomDat^.Save(mdateiname);
       {******************************** Ende Bearbeiten einer Klasse}

      end;

     end;
    end;
   Close(f);
   dispose(v,done);
   dispose(z,done);
   dispose(MomDat,done);
   end;
 end;


Function RunRdbMomMat(StiDsc:StiDsc_T; RdbDat:RdbDat_P; RdbTyp:RdbTyp_T; PslDat:PslDat_T; MomDsc:MomDsc_T):Word;

 Var
  f:          Text;
  mdateiname: PathStr;
  zeile:      String;
  SmplCls:    Char;
  Fertig:     Boolean;
  ClsCount:   Word;
  i,j:        Word;
  Cls:        Char;
  v,r,z:      PMatrix;
  MomDat:     MomDat_P;

 begin
 RunRdbMomMat:=0;
 v:=New(PMatrix,Init('V',StiDsc.LengthMv,1));
 r:=New(PMatrix,Init('R',RdbDat^.LengthPv,1));
 z:=New(PMatrix,Init('Z',PslDat.LengthPv+1,1));
 MomDat:=New(MomDat_P,Init('Mom',PslDat.LengthPv+1,PslDat.LengthPv+1,PslDat,' ',0));

 with StiDsc do
   begin
   Assign(f,StiDatei); {$I-} Reset(f); {$I+}
   if not (IOResult=0) then RunRdbMomMat:=1 else
    begin
    for i:=32 to 255 do
     begin
     Cls:=Char(i);
     if Cls in Classes then

      {************************************** Bearbeiten einer Klasse}
      begin
      Reset(f); Fertig:=False; ClsCount:=0; MomDat^.Matrix^.Initialize;
      while not Fertig do
       begin
       readln(f,zeile); GetPar(zeile);
       if ParStr(PosLabel)=Cls then
          begin
          Inc(ClsCount);
          Fertig:= EOF(f) or (ClsCount>=StiDsc.SmplPCls);
          for j:=0 to LengthMv-1 do v^.WriteMatrixElement(j,0,ValR(ParStr(UseVon+j)));

          {************************************** Verarbeitung Sample}
          RadBasVToX(RdbDat,RdbTyp,v,r);
          VtoZ(PslDat,r,z);
          AkkuMom(z,MomDat);
          {********************************* Ende Verarbeitung Sample}

          end;
       end;

       CmplMom(MomDat);
       mdateiname:=MomDsc.MomDatei+Cls+'.zmm';

       MomDat^.Class:=Cls;
       MomDat^.Count:=ClsCount;
       MomDat^.Save(mdateiname);
       {******************************** Ende Bearbeiten einer Klasse}

      end;

     end;
    end;
   Close(f);
   dispose(v,done);
   dispose(r,done);
   dispose(z,done);
   dispose(MomDat,done);
   end;
 end;



Function GetRezept(Var Rezept:Rezept_T):Word;
 Var
  rf:         Text;
  zeile:      string;
  s:          string;
  Cls:        Char;
  Wert,Sum:   Extended;
  MDateiName: PathStr;
  k:          Word;
  found:      Boolean;

 begin
  GetRezept:=0;
  with rezept do
   begin
   Assign(rf,RezeptDatei); {$I-} Reset(rf); {$I+}
   if not IOResult=0 then GetRezept:=1 else
    begin
    Sum:=0;
    NumbOfContrib:=0; NumbOfClasses:=0;
    while not EOF(rf) do
     begin
     readln(rf,zeile); GetPar(zeile); s:=ParStr(1);
     if (ParCount=3) and (s[1]<>'*') then
      begin
      Inc(NumbOfContrib); Cls:=s[1]; Wert:=ValR(ParStr(2));

      k:=0; found:=False;
      while not found and (k<NumbOfClasses) do
       begin
       Inc(k);
       found:=(Classes[k]=Cls);
       end;
      if found then BelongsTo[NumbOfContrib]:=k
               else begin
                    Inc(NumbOfClasses);
                    Classes[NumbOfClasses]:=Cls;
                    BelongsTo[NumbOfContrib]:=NumbOfClasses;
                    end;

      ClsLabel[NumbOfContrib]:=   Cls;
      Share[NumbOfContrib]:=      Wert;
      MDateiName[NumbOfContrib]:= ParStr(3);
      Sum:=Sum+Share[NumbOfContrib];
      end;
     end;
    for k:=1 to NumbOfClasses do Share[k]:=Share[k]/Sum;
    end;
  Close(rf);
  end;
 end;


Function Rgr(psldat:PslDat_T; classes:ClsTab_T; RgrDsc:RgrDsc_T; Var Mat:PMatrix):Word;
 Type
  Stat_T =   (elim,avail,used,target);
 Var
  pf:         Text;
  Status:     array[0..256] of Stat_T;
  ReconErr:   Extended;
  fak:        Extended;
  RelRecon:   Extended;
  ResidVar:   Extended;
  Fertig:     Boolean;
  i:          Integer;
  n:          Word;
  NrKlfSave:  Word;
  kdateiname: String;

 procedure StatusInit;
  Var
   i:Word;
  begin
  for i:=0 to RgrDsc.AnzX-1 do Status[i]:=avail;
  for i:=RgrDsc.AnzX to RgrDsc.AnzX+RgrDsc.AnzY-1 do Status[i]:=target;
  end;

 procedure Trace;
  Var
   i:Word; wert:Extended;
  begin
  ReconErr:=0; ResidVar:=0;
  for i:=0 to RgrDsc.AnzX+RgrDsc.AnzY-1 do
   begin
   Mat^.ReadMatrixElement(i,i,wert);
   case Status[i] of
    avail:  ReconErr:=ReconErr+wert;
    target: ResidVar:=ResidVar+wert;
   end;
   end;
  end;

 function Select:Word;
  Var
   i,j:                            Word;
   Diag,MDiag,DeltaS,MDeltaS,wert: Extended;
   Found:                          Boolean;
  begin
  MDiag:=0; MDeltaS:=0; Fertig:=True; Found:=False; i:=0;
  while not Found and (i<RgrDsc.AnzX) do
   begin
   if Status[i]=avail then
    begin
    Mat^.ReadMatrixElement(i,i,Diag);
    if Diag<RgrDsc.MinDiag then Status[i]:=elim
     else
      begin
      Fertig:=False;
      case RgrDsc.Strt of
       strtno: begin Found:=True; Select:=i; end;
       strtlu: if Diag>MDiag then begin MDiag:=Diag; Select:=i; end;
       strtms: begin
               DeltaS:=0;
               for j:=1 to RgrDsc.AnzY do
                begin
                Mat^.ReadMatrixElement(i,RgrDsc.AnzX+j-1,wert);
                DeltaS:=DeltaS+sqr(wert);
                end;
               DeltaS:=DeltaS/Diag;
               if DeltaS>MDeltaS then begin MDeltaS:=DeltaS; Select:=i; end;
               end;
      end;
      end;
    end;
   Inc(i);
   end;
  end;

 procedure RegreSchritt(n:Word);
  Var
   pivot,fak,wert,werthzeile: Extended;
   i,j,nn:Word;
  begin
  nn:=RgrDsc.AnzX+RgrDsc.AnzY;
  mat^.ReadMatrixElement(n,n,wert);
  pivot:=1/wert;

  for j:=0 to nn-1 do
   if not ((Status[n]=elim) or (Status[n]=used)) then
    begin
    mat^.ReadMatrixElement(n,j,wert);
    mat^.WriteMatrixElement(n,j,wert*pivot);
    end;

  for i:=0 to nn-1 do
   if not ((Status[i]=elim) or (i=n)) then
    begin
    mat^.ReadMatrixElement(i,n,fak);
    for j:=0 to nn-1 do
     if not ((Status[j]=elim) or (Status[j]=used)) then
      begin
      mat^.ReadMatrixElement(n,j,werthzeile);
      mat^.ReadMatrixElement(i,j,wert);
      mat^.WriteMatrixElement(i,j,wert-fak*werthzeile);
      end;
    end;

  Status[n]:=used;
  end;


 procedure Ablegen(psldat:PslDat_T; classes:ClsTab_T);
  Var
   klf:        ClfDat_P;
   m:          Pmatrix;
   nullzeile:  PMatrix;
   kdateiname: String;
   i:          Word;
  begin
  Inc(NrKlfSave);
  klf:=new(ClfDat_P,Init('Klf',RgrDsc.AnzX,RgrDsc.AnzY,psldat,RgrDsc.AnzY,classes));

  Matrix_Grab(Mat,0,RgrDsc.AnzX,RgrDsc.AnzX,RgrDsc.AnzY,m);

  nullzeile:=new(PMatrix,Init('nullz',1,RgrDsc.AnzY));
  for i:=0 to RgrDsc.AnzX-1 do
   if not (Status[i]=used) then m^.Insert(nullzeile,i,0);
  dispose(nullzeile,done);

  klf^.Matrix^.FillWith(m);
  dispose(m,done);

  kdateiname:=RgrDsc.KlfDatei+Trim(StrI(NrKlfSave,2))+'.zkq';

  {writeln('Schreibe Zwischenresultat-Klassifikator ',kdateiname); }

  klf^.Save(kdateiname);
  dispose(klf,done);
  end;


 begin
 Rgr:=0; NrKlfSave:=0;
 Assign(pf,RgrDsc.PrtDatei); {$I-} Rewrite(pf); {$I+}
 if not IOResult=0 then Rgr:=2
  else
   begin
   Fertig:=False; i:=-1;
   writeln(pf,'* Regre-Protokoll (',RgrDsc.DscDatei,')');
   writeln(pf,'* Nr   RestStr   RelReko  Slct');
   writeln(pf,'*-----------------------------');
   StatusInit;
   Trace; fak:=1/ReconErr;
   writeln(pf,i:4,ResidVar:10:4,ReconErr*fak:10:4);
   if RgrDsc.X0First then n:=0 else n:=Select;
   while not Fertig do begin
                       Inc(i);
                       RegreSchritt(n);
                       Trace;
                       if (i mod RgrDsc.StepPrt)=0 then
                        writeln(pf,i:4,ResidVar:10:4,ReconErr*fak:10:4,n:6);
                       if (i>0) and ((i mod RgrDsc.StepKlf)=0) then
                        Ablegen(psldat,classes);
                       n:=Select;
                       end;
   Ablegen(psldat,classes);
   Close(pf);
   end;
 end;


Function RunMkQMK(Rezept:Rezept_T; RgrDsc:RgrDsc_T):Word;
 Var
  MomDat:     MomDat_P;
  psldat:     PslDat_T;
  Mat:        PMatrix;
  m:          PMatrix;
  AnzX,AnzY:  Word;
  k,j:        Word;
  alt,neu:    Extended;


 begin
 RunMkQMK:=0;

 with Rezept do
  begin
  k:=1; j:=BelongsTo[k];
  MomDat:=new(MomDat_P,InitLoad(MDateiName[k]));
  AnzX:=MomDat^.Matrix^.col-1; AnzY:=NumbOfClasses;
  psldat:=MomDat^.PslDat;

  Mat:=new(PMatrix,Init('Mat',AnzX+AnzY,AnzX+AnzY));

  MomDat^.Matrix^.MultWithFactor(Share[k]);
  Matrix_Grab(MomDat^.Matrix,0,0,AnzX,AnzX,m);
  Mat^.Insert(m,0,0);
  dispose(m,done);

  Matrix_Grab(MomDat^.Matrix,0,AnzX,AnzX,1,m);
  Mat^.Insert(m,0,AnzX+j-1);
  dispose(m,done);

  Matrix_Grab(MomDat^.Matrix,AnzX,0,1,AnzX,m);
  Mat^.Insert(m,AnzX+j-1,0);
  dispose(m,done);

  MomDat^.Matrix^.ReadMatrixElement(AnzX,AnzX,neu);
  Mat^.WriteMatrixElement(AnzX+j-1,AnzX+j-1,neu);
  dispose(MomDat,done);

  for k:=2 to NumbOfContrib do
   begin
   MomDat:=new(MomDat_P,InitLoad(MDateiName[k]));
   j:=BelongsTo[k];
   if (MomDat^.Matrix^.col<>AnzX+1)              then RunMkQMK:=3 else
   if (MomDat^.Psldat.LengthPv<>PslDat.LengthPv) then RunMkQMK:=4
    else
     begin
     MomDat^.Matrix^.MultWithFactor(Share[k]);

     Matrix_Grab(MomDat^.Matrix,0,0,AnzX,AnzX,m);
     Mat^.AddInsert(m,0,0);
     dispose(m,done);

     Matrix_Grab(MomDat^.Matrix,0,AnzX,AnzX,1,m);
     Mat^.AddInsert(m,0,AnzX+j-1);
     dispose(m,done);

     Matrix_Grab(MomDat^.Matrix,AnzX,0,1,AnzX,m);
     Mat^.AddInsert(m,AnzX+j-1,0);
     dispose(m,done);

     Mat^.ReadMatrixElement(AnzX+j-1,AnzX+j-1,alt);
     MomDat^.Matrix^.ReadMatrixElement(AnzX,AnzX,neu);
     neu:=alt+neu;
     Mat^.WriteMatrixElement(AnzX+j-1,AnzX+j-1,neu);
     dispose(MomDat,done);
     end;
   end;

  RgrDsc.AnzX:=AnzX;
  RgrDsc.AnzY:=AnzY;
  end;

 Rgr(psldat,rezept.classes,RgrDsc,Mat);

 dispose(Mat,done);
 end;


Function RunClsf(StiDsc:StiDsc_T; klfdatei,dvektdatei:PathStr):Word;

 Var
  vf,df:         Text;
  Klf:           ClfDat_P;
  Psl:           PslDat_T;
  LengthPv:      Word;
  v,x,d:         PMatrix;
  NumbOfClasses: Word;
  Koeff:         PMatrix;
  zeile:         String;
  s:             String;
  Cls:           Char;
  Ident:         Word;
  j:             Word;
  w:             Extended;


 begin
 RunClsf:=0;

 Klf:=new(ClfDat_P,InitLoad(klfdatei));
 LengthPv:=Klf^.Matrix^.row;
 NumbOfClasses:=Klf^.NumbOfClasses;
 Psl:=Klf^.PslDat;
 Koeff:=new(PMatrix,InitCopy(Klf^.Matrix));
 Koeff^.Transpose;
 dispose(Klf,done);

 v:=New(PMatrix,Init('V',StiDsc.LengthMv,1));
 x:=New(PMatrix,Init('X',LengthPv,1));

 Assign(df,dvektdatei); {$I-} Rewrite(df); {$I+}
 if not (IOResult=0) then RunClsf:=2 else
  begin


  with StiDsc do
   begin
   Assign(vf,StiDatei); {$I-} Reset(vf); {$I+}
   if not (IOResult=0) then RunClsf:=1 else
    begin

    while not Eof(vf) do
     begin
     readln(vf,zeile); GetPar(zeile);
     if PosLabel > 0 then begin
                          s:=    ParStr(PosLabel);
                          Cls:=  s[1];
                          end;
     if PosIdent > 0 then Ident:=Vali(ParStr(PosIdent));
     for j:=0 to LengthMv-1 do v^.WriteMatrixElement(j,0,ValR(ParStr(UseVon+j)));

     {************************************** Verarbeitung Sample}
     VtoX(Psl,v,x);
     Matrix_Mult(Koeff,x,d);

     write(df,Cls:1,Ident:12);
     for j:=0 to NumbOfClasses-1 do
      begin
      d^.ReadMatrixElement(j,0,w);
      write(df,w:12:6);
      end;
     writeln(df);

     dispose(d,done);

     {********************************* Ende Verarbeitung Sample}

     end;


    Close(vf);
    end;

   end;
  Close(df);
  end;

 dispose(v,done);
 dispose(x,done);
 dispose(Koeff,done);
 end;



Function RunRdbClsf(StiDsc:StiDsc_T; RdbDat:RdbDat_P; RdbTyp:RdbTyp_T; klfdatei,dvektdatei:PathStr):Word;

 Var
  vf,df:         Text;
  Klf:           ClfDat_P;
  Psl:           PslDat_T;
  LengthPv:      Word;
  v,r,x,d:       PMatrix;
  NumbOfClasses: Word;
  Koeff:         PMatrix;
  zeile:         String;
  s:             String;
  Cls:           Char;
  Ident:         Word;
  j:             Word;
  w:             Extended;
  sum:           Extended;


 begin
 RunRdbClsf:=0;

 Klf:=new(ClfDat_P,InitLoad(klfdatei));
 LengthPv:=Klf^.Matrix^.row;
 NumbOfClasses:=Klf^.NumbOfClasses;
 Psl:=Klf^.PslDat;
 Koeff:=new(PMatrix,InitCopy(Klf^.Matrix));
 Koeff^.Transpose;
 dispose(Klf,done);

 v:=New(PMatrix,Init('V',StiDsc.LengthMv,1));
 r:=New(PMatrix,Init('R',RdbDat^.LengthPv,1));
 x:=New(PMatrix,Init('X',LengthPv,1));

 Assign(df,dvektdatei); {$I-} Rewrite(df); {$I+}
 if not (IOResult=0) then RunRdbClsf:=2 else
  begin


  with StiDsc do
   begin
   Assign(vf,StiDatei); {$I-} Reset(vf); {$I+}
   if not (IOResult=0) then RunRdbClsf:=1 else
    begin

    while not Eof(vf) do
     begin
     readln(vf,zeile); GetPar(zeile);
     if PosLabel > 0 then begin
                          s:=    ParStr(PosLabel);
                          Cls:=  s[1];
                          end;
     if PosIdent > 0 then Ident:=Vali(ParStr(PosIdent));
     for j:=0 to LengthMv-1 do v^.WriteMatrixElement(j,0,ValR(ParStr(UseVon+j)));

     {************************************** Verarbeitung Sample}
     RadBasVToX(RdbDat,RdbTyp,v,r);
     VtoX(Psl,r,x);
     Matrix_Mult(Koeff,x,d);

     if RdbTyp=clcprb then
      begin
      sum:=0;
      for j:=0 to NumbOfClasses-1 do begin
                                     d^.ReadMatrixElement(j,0,w);
                                     sum:=sum+w;
                                     end;
      end;

     write(df,Cls:1,Ident:12);
     for j:=0 to NumbOfClasses-1 do
      begin
      d^.ReadMatrixElement(j,0,w);
      if RdbTyp=clcprb then write(df,w/sum:12:6) else write(df,w:12:6);
      end;
     writeln(df);

     dispose(d,done);

     {********************************* Ende Verarbeitung Sample}

     end;


    Close(vf);
    end;

   end;
  Close(df);
  end;

 dispose(v,done);
 dispose(r,done);
 dispose(x,done);
 dispose(Koeff,done);
 end;


function RadBasVtoX(RdbDat:RdbDat_P; RdbTyp:RdbTyp_T; v:PMatrix; Var x:PMatrix):Word;
 Var
  i:           Word;
  w:           Extended;
  m1,m2,m3,m4: PMatrix;

 begin
 RadBasVtoX:=0;
 with RdbDat^ do
  begin
  for i:=0 to LengthPv-1 do
   begin
   m1:=New(PMatrix,InitCopy(v));
   m1^.Subtract(cen[i+1]);
   m2:=New(PMatrix,InitCopy(m1));
   m2^.Transpose;
   Matrix_Mult(kovi[i+1],m1,m3);
   Matrix_Mult(m2,m3,m4);
   dispose(m1,done);
   dispose(m2,done);
   dispose(m3,done);
   m4^.ReadMatrixElement(0,0,w);
   dispose(m4,done);
   w:=exp(-1/2*w);
   if rdbtyp=clcprb then w:=w*sqrtdeti[i+1];
   x^.WriteMatrixElement(i,0,w);
   end;
  end;
 end;


function RunRdb(StiDsc:StiDsc_T; RdbDat:RdbDat_P; RdbTyp:RdbTyp_T; ErgStiDatei:PathStr):Word;
 Var
  vf,xf:        Text;
  zeile,s:      String;
  Cls:          Char;
  Ident:        Word;
  v,x:          PMatrix;
  i:            Word;
  w:            Extended;



 begin
 RunRdb:=0;

 v:=New(PMatrix,Init('V',2,1));
 x:=New(PMatrix,Init('X',RdbDat^.LengthPv,1));

 Assign(vf,StiDsc.StiDatei); {$I-} Reset(vf); {$I+}
 if not (IOResult=0) then RunRdb:=1 else
  begin
  Assign(xf,ErgStiDatei); {$I-} Rewrite(xf); {$I+}
  if not (IOResult=0) then RunRdb:=2 else
   begin
   while not Eof(vf) do
    begin
    {************************************** Verarbeitung Sample}
    readln(vf,zeile); GetPar(zeile);

    if StiDsc.PosLabel > 0 then begin
                                s:=    ParStr(StiDsc.PosLabel);
                                Cls:=  s[1];
                                end;
    if StiDsc.PosIdent > 0 then Ident:=Vali(ParStr(StiDsc.PosIdent));
    v^.WriteMatrixElement(0,0,ValR(ParStr(StiDsc.UseVon  )));
    v^.WriteMatrixElement(1,0,ValR(ParStr(StiDsc.UseVon+1)));
    RadBasVToX(RdbDat,RdbTyp,v,x);

    write(xf,cls:1,ident:8);
    for i:=0 to RdbDat^.LengthPv-1 do
     begin
     x^.ReadMatrixElement(i,0,w);
     write(xf,w:12:6);
     end;
    writeln(xf);

    end;
   Close(xf);
   end;
  Close(vf);
  dispose(x,done);
  dispose(v,done);
  end;
 end;



begin
end.
