unit Matrix;

interface

uses Objects,crt;

type PMatrix    = ^TMatrix;
     TMatrix    = object(TObject)
                   name   :string[32];
                   row,col:word;
                   data   :pointer;
                   intern :word;        (* Finger weg!!!!!! *)

                   constructor Init(_name:string;_row,_col:word);
                   constructor InitLoad(s:string);
                   constructor InitCopy(m:PMatrix);
                   destructor Done; virtual;

                   function ReadMatrixElement(r,c:integer; var wert:extended):word; virtual;
                   function WriteMatrixElement(r,c:integer; wert:extended):word; virtual;
                   function SwapElement(r1,c1,r2,c2:word):word; virtual;
                   function ChangeDimension(_z,_s:word):word; virtual;
                   function AddElement(r,c:integer; wert:extended):word; virtual;
                   function SubElement(r,c:integer; wert:extended):word; virtual;
                   function MultElement(r,c:integer; wert:extended):word; virtual;
                   function DivElement(r,c:integer; wert:extended):word; virtual;

                   function Initialize:word; virtual;
                   function FillWith(m:PMatrix):word; virtual;
                   function Insert(m:PMatrix;z,s:word):word; virtual;
                   function AddInsert(m:PMatrix;z,s:word):word; virtual;
                   function LoadFromFile(s:string):word; virtual;

                   function Save(s:string):word; virtual;
                   function Load(s:string):word; virtual;

                   function Identity:word; virtual;
                   function Add(summand:PMatrix):word; virtual;
                   function Subtract(minor:PMatrix):word; virtual;
                   function MultWithFactor(factor:extended):word; virtual;
                   function Transpose:word;virtual;
                   function Invert:word;virtual;

                   function Print(nachkomma:word):word;
                   function ChangeName(_name:string):word;
                  end;

var AllMatrices : array[0..400] of PMatrix;
    NoOfMatrices: word;

const AbortOnMatrixError:boolean = true;

function Matrix_Add     (a,b:PMatrix;var erg:PMatrix):word;
function Matrix_Subtract(a,b:PMatrix;var erg:PMatrix):word;
function Matrix_Mult    (a,b:PMatrix;var erg:PMatrix):word;

function Matrix_Invert(a:PMatrix;var erg:PMatrix):word;
function Matrix_Transpose(a:PMatrix;var erg:PMatrix):word;

function Matrix_Grab(a:PMatrix;z,s,h,b:word;var erg:PMatrix):word;
function Matrix_LUP(a:PMatrix;var L,U,P:PMatrix):word;
function Matrix_Solve(A,b:PMatrix;var x:PMatrix):word;
function Matrix_Det(A:PMatrix;var det:extended):word;
function Matrix_Spur(A:PMatrix;var spur:extended):word;

function Matrix_Balance(a:PMatrix; var erg:PMatrix):word;
function Matrix_Hessenberg(a:PMatrix; var erg:PMatrix):word;
function Matrix_EigenValues(a:PMatrix;var re:PMatrix;var im:PMatrix):word;

function Matrix_Active:word;

function Matrix_Error(i:word;header:string):word;
function MatrixErrorString(i:word):string;

(* Zum vollstaendigen Entfernen der Matrizen aus dem Speicher *)
procedure MatrixExitProcedure;
var SaveExitProc : Pointer;


implementation

function SIGN(a,b:extended):extended;
begin
if b>=0 then SIGN:=abs(a) else SIGN:=-abs(a);
end;


constructor TMatrix.Init(_name:string;_row,_col:word);
(****************************************************)
(* Initialisiert eine Matrix mit Name und Groesse   *)
(****************************************************)
begin
 row:=_row;
 col:=_col;
 name:=_name;
 intern:=NoOfMatrices;        (* Nummer der Matrix *)
 GetMem(data,row*col*10);
 AllMatrices[NoOfMatrices]:=@Self;  (* Uebernahme in die Hauptliste *)
 inc(NoOfMatrices);
 Initialize;
end;

constructor TMatrix.InitLoad(s:string);
(****************************************************)
(* Initialisiert eine Matrix von Platte             *)
(****************************************************)
begin
 Init('_INIT',1,1);
 Load(s);
end;

constructor TMatrix.InitCopy(m:PMatrix);
(**************************************)
(* Initialsiert Matrix durch Kopie    *)
(**************************************)
begin
 Init(m^.name,m^.row,m^.col);
 FillWith(m);
end;

destructor TMatrix.Done;
(**********************)
(* Entfernt Matrix    *)
(**********************)
var i:integer;
begin
 FreeMem(data,row*col*10);
 for i:=intern+1 to NoOfMatrices-1 do
  begin
   AllMatrices[i-1]:=AllMatrices[i];     (* um eins nach unten schieben *)
   dec(AllMatrices[i-1]^.intern);        (* und Nummer dekrementieren   *)
  end;
 dec(NoOfMatrices);
end;

function TMatrix.ReadMatrixElement(r,c:integer; var wert:extended):word;
(**********************************************************************)
(* ReadMatrixElement liest ein Matrixelement und liefert wert zurueck *)
(**********************************************************************)
var s,o:word;
    index:word;
    quelle:^extended;
begin
 if (c>=0) and (c<col) and (r>=0) and (r<row) then
  begin
   ReadMatrixElement:=0;
   index:=(col*r+c)*10;
   s:=seg(data^);
   o:=ofs(data^);
   quelle:=ptr(s,o+index);
   wert:=quelle^;
  end
 else
  begin
   ReadMatrixElement:=Matrix_Error(1,'ReadMatrixElement: '+name);
  end;
end;

function TMatrix.SwapElement(r1,c1,r2,c2:word):word;
(**********************************)
(* Vertauscht zwei Eintraege      *)
(**********************************)

var wert1,wert2:extended;
begin
 readmatrixelement(r1,c1,wert1);
 readmatrixelement(r2,c2,wert2);
 writematrixelement(r1,c1,wert2);
 writematrixElement(r2,c2,wert1);
end;

function TMatrix.ChangeDimension(_z,_s:word):word;
begin
 if row*col<>_z*_s then ChangeDimension:=Matrix_Error(11,'ChangeDimension: '+name)
 else
  begin
   col:=_z;
   row:=_s;
  end;
end;

function TMatrix.AddElement(r,c:integer; wert:extended):word;
(***************************************************)
(* Addiert auf ein Element drauf                   *)
(***************************************************)
var w:extended;
begin
 ReadMatrixElement(r,c,w);
 WriteMatrixElement(r,c,w+wert);
end;

function TMatrix.SubElement(r,c:integer; wert:extended):word;
(***************************************************)
(* Zieht von einem Element einen Wert ab           *)
(***************************************************)
var w:extended;
begin
 ReadMatrixElement(r,c,w);
 WriteMatrixElement(r,c,w-wert);
end;

function TMatrix.MultElement(r,c:integer; wert:extended):word;
var w:extended;
(***************************************************)
(* Multipliziert ein Element mit wert              *)
(***************************************************)
begin
 ReadMatrixElement(r,c,w);
 WriteMatrixElement(r,c,w*wert);
end;

function TMatrix.DivElement(r,c:integer; wert:extended):word;
var w:extended;
(***************************************************)
(* Teilt ein Element durhc einen wert              *)
(***************************************************)
begin
 ReadMatrixElement(r,c,w);
 WriteMatrixElement(r,c,w/wert);
end;


function TMatrix.WriteMatrixElement(r,c:integer; wert:extended):word;
(*******************************************************************)
(* WriteMatrixElement besetzt ein Element der Matrix mit wert      *)
(*******************************************************************)
var s,o:word;
    index:word;
    ziel:^extended;
begin
 if (c>=0) and (c<col) and (r>=0) and (r<row) then
  begin
   WriteMatrixElement:=0;
   index:=(col*r+c)*10;
   s:=seg(data^);
   o:=ofs(data^);
   ziel:=ptr(s,o+index);
   ziel^:=wert;
  end
 else
  begin
   WriteMatrixElement:=Matrix_Error(1,'WriteMatrixElement: '+name);
  end;
end;

function TMatrix.Initialize:word;
(*******************************)
(* Legt Nullmatrix an          *)
(*******************************)
var i,j:word;
begin
 for i:=0 to row-1 do
  begin
   for j:=0 to col-1 do
    begin
     WriteMatrixElement(i,j,0);
    end;
  end;
 Initialize:=0;
end;

function TMatrix.FillWith(m:PMatrix):word;
(****************************************)
(* Kopiert Matrix m auf die Matrix      *)
(****************************************)
var i,j:word;
    wert:extended;
begin
 if (col=m^.col) and (row=m^.row) then
  begin
   for i:=0 to row-1 do
    begin
     for j:=0 to col-1 do
      begin
       m^.ReadMatrixElement(i,j,wert);
       WriteMatrixElement(i,j,wert);
      end;
    end;
   FillWith:=0
  end
 else FillWith:=Matrix_Error(3,'FillWith: '+name);
end;

function TMatrix.Insert(m:PMatrix;z,s:word):word;
(***********************************************)
(* Fuegt eine Matrix irgendwo in eine andere   *)
(***********************************************)
var i,j:word;
    wert:extended;
begin
 if (m^.row+z>row) or (m^.col+s>col) then Insert:=Matrix_Error(8,'Insertion: '+name)
 else
  begin
   Insert:=0;
   for i:=0 to m^.row-1 do
    begin
     for j:=0 to m^.col-1 do
      begin
       m^.ReadMatrixElement(i,j,wert);
       WriteMatrixElement(z+i,s+j,wert);
      end;
    end;
  end;
end;


function TMatrix.AddInsert(m:PMatrix;z,s:word):word;
(**************************************************)
(*          Fuegt Teilmatrix additiv ein          *)
(**************************************************)
var i,j:     word;
    alt,neu: extended;
begin
 if (m^.row+z>row) or (m^.col+s>col) then AddInsert:=Matrix_Error(8,'Insertion: '+name)
 else
  begin
   AddInsert:=0;
   for i:=0 to m^.row-1 do
    begin
     for j:=0 to m^.col-1 do
      begin
       ReadMatrixElement(z+i,s+j,alt);
       m^.ReadMatrixElement(i,j,neu);
       neu:=alt+neu;
       WriteMatrixElement(z+i,s+j,neu);
      end;
    end;
  end;
end;



function TMatrix.LoadFromFile(s:string):word;
(*******************************************)
(* Laedt Matrix von File, Groesse bekannt  *)
(*******************************************)
var f:text;
    i,j:word;
    wert:extended;
begin
 assign(f,s);
 reset(f);
 for i:=0 to row-1 do
  begin
   for j:=0 to col-1 do
    begin
     read(f,wert);
     WriteMatrixElement(i,j,wert);
    end;
   readln(f);
  end;
end;

function TMatrix.Save(s:string):word;
(***********************************)
(* Sichert die Matrix kompakt ab   *)
(***********************************)
var f:file;
begin
 assign(f,s);
 rewrite(f,1);
 blockwrite(f,name,33);
 blockwrite(f,row,2);
 blockwrite(f,col,2);
 blockwrite(f,data^,row*col*10);
 close(f);
end;

function TMatrix.Load(s:string):word;
(***********************************)
(* Laedt Matrix von Platte         *)
(***********************************)
var f:file;
begin
 assign(f,s);
 reset(f,1);
 blockread(f,name,33);
 FreeMem(data,row*col*10);
 blockread(f,row,2);
 blockread(f,col,2);
 GetMem(data,row*col*10);
 blockread(f,data^,row*col*10);
 close(f);
end;


function TMatrix.Identity:word;
(*****************************)
(* Schreibt 1en auf Diagonale*)
(*****************************)
var i:word;
begin
 if row<>col then
  begin
   Identity:=Matrix_Error(2,'Identity: '+name)
  end
 else
  begin
   Identity:=0;
   Initialize;
   for i:=0 to col-1 do
    begin
     WriteMatrixElement(i,i,1);
    end;
  end;
end;

function TMatrix.Add(summand:PMatrix):word;
(*****************************************)
(* Addiert zur Matrix die Matrix summand *)
(*****************************************)
var i,j:word;
    wert1,wert2:extended;
begin
 if (col=summand^.col) and (row=summand^.row) then
  begin
   Add:=0;
   for i:=0 to row-1 do
    begin
     for j:=0 to col-1 do
      begin
       ReadMatrixElement(i,j,wert1);
       summand^.ReadMatrixElement(i,j,wert2);
       WriteMatrixElement(i,j,wert1+wert2);
      end;
    end;
  end
 else
  begin
   Add:=Matrix_Error(3,'Addition: '+name);
  end;
end;


function TMatrix.Subtract(minor:PMatrix):word;
(*****************************************)
(* Addiert zur Matrix die Matrix summand *)
(*****************************************)
var i,j:word;
    wert1,wert2:extended;
begin
 if (col=minor^.col) and (row=minor^.row) then
  begin
   Subtract:=0;
   for i:=0 to row-1 do
    begin
     for j:=0 to col-1 do
      begin
       ReadMatrixElement(i,j,wert1);
       minor^.ReadMatrixElement(i,j,wert2);
       WriteMatrixElement(i,j,wert1-wert2);
      end;
    end;
  end
 else
  begin
   Subtract:=Matrix_Error(3,'Subtraction: '+name);
  end;
end;


function TMatrix.MultWithFactor(factor:extended):word;
(*****************************************)
(* Addiert zur Matrix die Matrix summand *)
(*****************************************)
var i,j:word;
    wert:extended;
begin
 for i:=0 to row-1 do
  begin
   for j:=0 to col-1 do
    begin
     ReadMatrixElement(i,j,wert);
     WriteMatrixElement(i,j,wert*factor);
    end;
  end;
 MultWithFactor:=0;
end;

function TMatrix.Transpose:word;
(******************************)
(*Transponiert eine Matrix    *)
(******************************)
var erg:PMatrix;
    temp:word;
begin
 Matrix_Transpose(@Self,erg);
 Move(erg^.Data^,Data^,col*row*10);
 Dispose(erg,Done);
 temp:=row;
 row:=col;
 col:=temp;
 if copy(name,length(name),1)='''' then name:=copy(name,1,length(name)-1)
                                   else name:=+name+'''';

end;

function TMatrix.Invert:word;
(**************************)
(* Invertiert sich        *)
(**************************)
var erg:PMatrix;
begin
 Matrix_Invert(@Self,erg);
 Move(erg^.Data^,Data^,col*row*10);
 Dispose(erg,Done);
 if copy(name,1,2)='1/' then name:=copy(name,3,30)
                        else name:='1/'+name;
end;

function TMatrix.Print(nachkomma:word):word;
(**************************)
(* Gibt Matrix aus        *)
(**************************)
var i,j,k:word;
    wert:extended;
    s:string;
    l:integer;
    maxlength:word;
    breite:word;

begin
 (* Laengstes Element suchen *)
 maxlength:=0;
 for i:=0 to row-1 do
  begin
   for j:=0 to col-1 do
    begin
     ReadMatrixElement(i,j,wert);
     str(trunc(wert),s);
     l:=length(s);
     if l>maxlength then
      begin
       maxlength:=l;
      end;
    end;
  end;
 breite:=col*(maxlength+nachkomma+2)+1;
 if breite+length(name)+1>78 then
  begin
   Print:=Matrix_Error(9,'Printing: '+name);
  end
 else
  begin
   writeln;
   for i:=1 to length(name)+1 do write(' ');write('');
   for i:=1 to breite do write(' ');write('');
   writeln;
   for i:=0 to row-1 do
    begin
     if i=((row-1) div 2) then write(name,'=')
                          else for k:=1 to length(name)+1 do write(' ');
     write(' ');
     for j:=0 to col-1 do
      begin
       ReadMatrixElement(i,j,wert);
       (* if wert>=0 then write(' '); *)
       write(wert:maxlength+nachkomma+1:nachkomma,' ');
      end;
     writeln('');
    end;
   for i:=1 to length(name)+1 do write(' ');write('');
   for i:=1 to breite do write(' ');write('');
   writeln;
   Print:=0;
  end;
end;
function TMatrix.ChangeName(_name:string):word;
begin
 name:=_name;
 ChangeName:=0;
end;

(***********************************************************************)

function Matrix_Add(a,b:PMatrix;var erg:PMatrix):word;
(****************************************************)
(* Addiert zwei Matrizen                            *)
(****************************************************)
begin
 if (a^.col=b^.col) and (a^.row=b^.row) then
  begin
   erg:=New(PMatrix,Init('_ADD',a^.row,a^.col));
   erg^.FillWith(a);
   erg^.Add(b);
   Matrix_Add:=0;
  end
 else Matrix_Add:=Matrix_Error(3,'Addition: '+a^.name+'+'+b^.name);
end;

function Matrix_Subtract(a,b:PMatrix;var erg:PMatrix):word;
(*********************************************************)
(* Macht a-b                                             *)
(*********************************************************)
begin
 if (a^.col=b^.col) and (a^.row=b^.row) then
  begin
   erg:=New(PMatrix,Init('_SUB',a^.row,a^.col));
   erg^.FillWith(a);
   erg^.Subtract(b);
   Matrix_Subtract:=0;
  end
 else Matrix_Subtract:=Matrix_Error(3,'Subtraction: '+a^.name+'-'+b^.name);
end;

function Matrix_Mult(a,b:PMatrix;var erg:PMatrix):word;
(*****************************************************)
(* Multipliziert a mit b                             *)
(*****************************************************)
var i,j,k:word;
    wert,wert1,wert2:extended;
begin
 if (a^.col=b^.row) then
  begin
   erg:=New(PMatrix,Init('_MULT',a^.row,b^.col));
   for i:=0 to a^.row-1 do
    begin
     for j:=0 to b^.col-1 do
      begin
       wert:=0;
       for k:=0 to a^.col-1 do
        begin
         a^.ReadMatrixelement(i,k,wert1);
         b^.ReadMatrixelement(k,j,wert2);
         wert:=wert+wert1*wert2;
        end;
       erg^.WriteMatrixElement(i,j,wert);
      end;
    end;
   Matrix_Mult:=0;
  end
 else Matrix_Mult:=Matrix_Error(4,'Multiplication: '+a^.name+'*'+b^.name);
end;


function Matrix_Invert(a:PMatrix;var erg:PMatrix):word;
(***************************)
(* Invertiert die Matrix   *)
(***************************)
var temp:PMatrix;
begin
 if a^.col<>a^.row then
  begin
   Matrix_Invert:=Matrix_Error(2,'Inversion: '+a^.name);
  end
 else
  begin
   temp:=New(PMatrix,Init('_temp',a^.row,a^.col));
   temp^.Identity;
   Matrix_Solve(a,temp,erg);
   Dispose(temp,done);
   erg^.ChangeName('_INVERT')
  end;
end;


function Matrix_Transpose(a:PMatrix;var erg:PMatrix):word;
(********************************************************)
(* Transponiert die axb-Matrix: ergibt bxa-Matrix       *)
(********************************************************)
var i,j:word;
    wert:extended;
begin
 erg:=New(PMatrix,Init('_TRANSP',a^.col,a^.row));
  for i:=0 to a^.row-1 do
  begin
   for j:=0 to a^.col-1 do
    begin
     a^.ReadMatrixElement(i,j,wert);
     erg^.WriteMatrixElement(j,i,wert);
    end;
  end;
end;

function Matrix_Grab(a:PMatrix;z,s,h,b:word;var erg:PMatrix):word;
(****************************************************************)
(* Kopiert eine Teilmatrix in eine andere Matrix                *)
(****************************************************************)
var i,j:word;
    wert:extended;
begin
 if (s+b>a^.col) or (z+h>a^.row) then Matrix_Grab:=Matrix_Error(8,'Grabbing: '+a^.name)
 else
  begin
   Matrix_Grab:=0;
   erg:=New(PMatrix,Init('_GRAB',h,b));
   for i:=0 to h-1 do
    begin
     for j:=0 to b-1 do
      begin
       a^.ReadMatrixElement(z+i,s+j,wert);
       erg^.WriteMatrixElement(i,j,wert);
      end;
    end;
  end;
end;

function Matrix_LUP(a:PMatrix;var L,U,P:PMatrix):word;
(************************)
(* Macht Lower-Upper-Mat*)
(************************)
var i,j,k,n:word;
    merk_k:word;
    wert,max:extended;
    third,second,first:extended;
    save:word;
    erg:PMatrix;
begin
 if a^.col<>a^.row then
  begin
   Matrix_LUP:=Matrix_Error(2,'LUP-Decomposition: '+a^.name);
  end
 else
  begin
   erg:=New(PMatrix,Init('_LUP',a^.col,a^.row));
   erg^.FillWith(a);
   P:=New(PMatrix,Init('_LUP_Perm',a^.col,a^.row));
   p^.Identity;
   Matrix_LUP:=0;
   n:=a^.col;
   for k:=0 to n-2 do
    begin
     max:=0;
      for i:=k to n-1 do
       begin
        erg^.ReadMatrixElement(i,k,wert);
        if abs(wert)>max then begin
                             max:=abs(wert);
                             merk_k:=i;
                            end;
       end;
     if max=0 then Matrix_LUP:=Matrix_Error(5,'LUP-Decomposition: '+a^.name);
     (* Spalten der Permuationsmatrix vertauschen *)
     for i:=0 to n-1 do
      begin
       P^.ReadMatrixElement(k,i,first);
       P^.ReadMatrixElement(merk_k,i,second);
       P^.WriteMatrixElement(k,i,second);
       P^.WriteMatrixElement(merk_k,i,first);
      end;

     for i:=0 to n-1 do
      begin
       erg^.ReadMatrixElement(k,i,first);
       erg^.ReadMatrixElement(merk_k,i,second);
       erg^.WriteMatrixElement(k,i,second);
       erg^.WriteMatrixElement(merk_k,i,first);
      end;
     for i:=k+1 to n-1 do
      begin
       erg^.ReadMatrixElement(i,k,first);
       erg^.ReadMatrixElement(k,k,second);
       erg^.WriteMatrixElement(i,k,first/second);
       for j:=k+1 to n-1 do
        begin
         erg^.ReadMatrixElement(i,j,first);
         erg^.ReadMatrixElement(i,k,second);
         erg^.ReadMatrixElement(k,j,third);
         erg^.WriteMatrixElement(i,j,first-second*third);
        end;
      end;
    end;
  end;
 L:=New(PMatrix,Init('_LUP_Lower',a^.col,a^.row));
 U:=New(PMatrix,Init('_LUP_Upper',a^.col,a^.row));
 L^.Identity;
 for i:=0 to n-1 do
  begin
   for j:=0 to i do
    begin
     erg^.ReadMatrixElement(j,i,wert);
     U^.WriteMatrixelement(j,i,wert);
    end;
   for j:=i+1 to n-1 do
    begin
     erg^.ReadMatrixElement(j,i,wert);
     L^.WriteMatrixelement(j,i,wert);
    end;
  end;

 Dispose(erg,Done);
end;

function Matrix_Solve(A,b:PMatrix;var x:PMatrix):word;
(****************************************************)
(* Loest lineares Gleichungssystem Ax=b (x auch Mat)*)
(****************************************************)
var L,U,P,y: PMatrix;
    n,m:word;
    i,j,k:integer;
    wert:extended;
    first,second:extended;
    po:word;
begin
 if (a^.col<>a^.row) then Matrix_Solve:=Matrix_Error(2,'Solving: '+a^.name+'*x='+b^.name)
 else if (b^.row<>a^.col) then Matrix_Solve:=Matrix_Error(7,'Solving: '+a^.name+'*x='+b^.name)
 else
  begin
   x:=New(PMatrix,Init('_x_solution',b^.row,b^.col));
   y:=New(PMatrix,Init('_y_solution',b^.row,b^.col));
   Matrix_LUP(A,L,U,P);
   Matrix_Solve:=0;

   n:=a^.row;
   m:=b^.col;

   for k:=0 to m-1 do
    begin
     for i:=0 to n-1 do
      begin
       wert:=0;
       for j:=0 to i-1 do
        begin
         l^.ReadMatrixElement(i,j,first);
         y^.ReadMatrixElement(j,k,second);
         wert:=wert+first*second;
        end;
       for j:=0 to n-1 do
        begin
         p^.ReadMatrixElement(i,j,first);
         if first=1 then po:=j;
        end;
       b^.ReadMatrixElement(po,k,first);
       y^.WriteMatrixElement(i,k,first-wert);
      end;
     for i:=n-1 downto 0 do
      begin
       wert:=0;
       for j:=i+1 to n-1 do
        begin
         u^.ReadMatrixElement(i,j,first);
         x^.ReadMatrixElement(j,k,second);
         wert:=wert+first*second;
        end;
       y^.ReadMatrixElement(i,k,first);
       wert:=first-wert;
       u^.ReadMatrixElement(i,i,first);
       if first=0 then Matrix_Solve:=Matrix_Error(5,'Solving: '+a^.name+'*x='+b^.name);
       x^.WriteMatrixElement(i,k,wert/first);
      end;
    end;
   Dispose(y,Done);
   Dispose(L,Done);
   Dispose(U,Done);
   Dispose(P,Done);
  end;
end;

function Matrix_Det(A:PMatrix;var det:extended):word;
(***************************************************)
(* Berechnet die Determinante einer Matrix         *)
(***************************************************)
var i,j:word;
    L,U,P:PMatrix;
    first:extended;
begin
 if a^.col<>a^.row then Matrix_Det:=Matrix_Error(2,'Determinante: '+a^.name)
 else
  begin
   Matrix_Det:=0;
   Matrix_LUP(A,L,U,P);
   det:=1;
   for i:=0 to a^.col-1 do
    begin
     U^.ReadMatrixElement(i,i,first);
     det:=det*first;
    end;
   Dispose(L,Done);
   Dispose(U,Done);
   Dispose(P,Done);
  end;
end;

function Matrix_Spur(A:PMatrix;var spur:extended):word;
(***************************************************)
(* Berechnet die Spur einer Matrix                 *)
(***************************************************)
var i,j:word;
    first:extended;
begin
 if a^.col<>a^.row then Matrix_Spur:=Matrix_Error(2,'Spur: '+a^.name)
 else
  begin
   Matrix_Spur:=0;
   spur:=0;
   for i:=0 to a^.col-1 do
    begin
     a^.ReadMatrixElement(i,i,first);
     spur:=spur+first;
    end;
  end;
end;

function Matrix_Balance(a:PMatrix; var erg:PMatrix):word;
(**************************************************)
(* erzeugt eine balncierte Matrix mit gleichen EW *)
(**************************************************)
const RADIX = 2;
var   n,i,j,last:integer;
      s,r,g,f,c,sqrdx,wert1,wert2:extended;
begin
 if a^.col<>a^.row then
  begin
   Matrix_Balance:=Matrix_Error(2,'Balance: '+a^.name);
  end
 else
  begin
   erg:=New(PMatrix,Init('Balance('+a^.name+')',a^.row,a^.col));
   erg^.FillWith(a);
   n:=a^.col;
   (* Beginn des eigentlichen Algorithmus *)
   sqrdx:=RADIX*RADIX;
   last:=0;
   while last=0 do
    begin
     last:=1;
     for i:=0 to n-1 do
      begin
       r:=0;
       c:=0;
       for j:=0 to n-1 do
        begin
         if j<>i then
          begin
           erg^.ReadMatrixElement(j,i,wert1);
           erg^.ReadMatrixElement(i,j,wert2);
           c:=c+abs(wert1);
           r:=r+abs(wert2);
          end;
        end;
       if (c<>0) and (r<>0) then
        begin
         g:=r/RADIX;
         f:=1;
         s:=c+r;
         while c<g do
          begin
           f:=f*RADIX;
           c:=c*sqrdx;
          end;
         g:=r*RADIX;
         while c>g do
          begin
           f:=f/RADIX;
           c:=c/sqrdx;
          end;
         if (c+r)/f < 0.95*s then
          begin
           last:=0;
           g:=1/f;
           for j:=0 to n-1 do
            begin
             erg^.ReadMatrixElement(i,j,wert1);
             erg^.WriteMatrixElement(i,j,wert1*g);
            end;
           for j:=0 to n-1 do
            begin
             erg^.ReadMatrixElement(j,i,wert1);
             erg^.WriteMatrixElement(j,i,wert1*f);
            end;
          end;
        end;
      end;
    end;
  end;
end;

function Matrix_Hessenberg(a:PMatrix; var erg:PMatrix):word;
(**********************************************************)
(* erzeugt aus einer quad. Matrix die Hessenberg-Form     *)
(**********************************************************)
var m,j,i : integer;
    y,x: extended;
    n: word;
    wert,wert2:extended;
begin
 if a^.col<>a^.row then
  begin
   Matrix_Hessenberg:=Matrix_Error(2,'Hessenberg: '+a^.name);
  end
 else
  begin
   erg:=New(PMatrix,Init('HB('+a^.name+')',a^.row,a^.col));
   erg^.FillWith(a);
   n:=a^.row;
   (* der algorithmus:*)
   for m:=1 to n-2 do
    begin
     x:=0;
     i:=m;
     for j:=m to n-1 do
      begin
       erg^.ReadMatrixElement(j,m-1,wert);
       if abs(wert)>abs(x) then
        begin
         x:=wert;
         i:=j;
        end;
      end;
     if i<>m then
      begin
       for j:=m-1 to n-1 do erg^.SwapElement(i,j,m,j);
       for j:=0 to n-1 do erg^.SwapElement(j,i,j,m);
      end;
     if x<>0 then
      begin
       for i:=m+1 to n-1 do
        begin
         erg^.ReadMatrixelement(i,m-1,y);
         if y<> 0 then
          begin
           y:=y/x;
           erg^.WriteMatrixElement(i,m-1,y);
           for j:=m to n-1 do
            begin
             erg^.ReadMatrixElement(m,j,wert);
             erg^.ReadMatrixElement(i,j,wert2);
             erg^.WriteMatrixElement(i,j,wert2-y*wert);
            end;
           for j:=0 to n-1 do
            begin
             erg^.ReadMatrixElement(j,i,wert);
             erg^.ReadMatrixElement(j,m,wert2);
             erg^.WriteMatrixElement(j,m,wert2+y*wert);
            end;
          end;
        end;
      end;
    end;
   for i:=0 to a^.row-1 do
    begin
     for j:=0 to a^.col-1 do
      begin
       if i>j+1 then erg^.writematrixelement(i,j,0);
      end;
    end;
  end;
end;

function Matrix_EigenValues(a:PMatrix;var re:PMatrix;var im:PMatrix):word;
var n,nn,m,l,k,j,its,i,mmin: integer;
    z,y,x,w,v,u,t,s,r,q,p,anorm: extended;
    wert1,wert2,wert3,wert: extended;
    c,b:PMatrix;
begin

 Matrix_Balance(a,c);
 Matrix_Hessenberg(c,b);
 c^.Done;
 re:=New(PMatrix,Init('Realteil',a^.row,1));
 im:=New(PMatrix,Init('Imaginrteil',a^.row,1));
 n:=a^.row;
 (* Der Algorithmus : *)
 b^.ReadMatrixElement(0,0,wert);
 anorm:=abs(wert);
 for i:=1 to n-1 do
  for j:=i-1 to n-1 do
   begin
    b^.ReadMatrixElement(i,j,wert);
    anorm:=anorm+abs(wert);
   end;
 nn:=n-1;
 t:=0;
 while nn>=0 do
  begin
   its:=0;
   repeat
    for l:=nn downto 1 do
     begin
      b^.ReadMatrixElement(l-1,l-1,wert1);
      b^.ReadMatrixElement(l,l,wert2);
      s:=abs(wert1)+abs(wert2);
      if s=0 then s:=anorm;
      b^.ReadMatrixElement(l,l-1,wert);
      if wert+s=s then break;
     end;
     b^.ReadMatrixElement(nn,nn,x);
     if l=nn then
      begin
       re^.WriteMatrixElement(nn,0,x+t);
       im^.WriteMatrixElement(nn,0,0);
       dec(nn);
      end
     else
      begin
       b^.ReadMatrixElement(nn-1,nn-1,y);
       b^.ReadMatrixElement(nn,nn-1,wert1);
       b^.ReadMatrixElement(nn-1,nn,wert2);
       w:=wert1*wert2;
       if l=nn-1 then
        begin
         p:=0.5*(y-x);
         q:=p*p+w;
         z:=sqrt(abs(q));
         x:=x+t;
         if q>0 then
          begin
           z:=p+sign(z,p);
           re^.WriteMatrixElement(nn-1,0,x+z);
           re^.WriteMatrixElement(nn  ,0,x+z);
           if z=0 then re^.WriteMatrixElement(nn,0,x-w/z);
           im^.WriteMatrixElement(nn-1,0,0);
           im^.WriteMatrixElement(nn  ,0,0);
          end
         else
          begin
           re^.WriteMatrixElement(nn-1,0,x+p);
           re^.WriteMatrixElement(nn  ,0,x+p);
           im^.WriteMatrixElement(nn-1,0,z);
           im^.WriteMatrixElement(nn  ,0,-z);
          end;
         nn:=nn-2;
        end
       else
        begin    (*No roots found .... *)
         if its=70 then Matrix_EigenValues:=Matrix_Error(10,'Eigenvalue: '+a^.name);
         if (its=10) or (its=20) then
          begin
           t:=t+x;
           for i:=0 to nn do
            begin
             b^.ReadMatrixElement(i,i,wert);
             b^.WriteMatrixElement(i,i,wert-x);
            end;
           b^.ReadMatrixElement(nn,nn-1,wert1);
           b^.ReadMatrixElement(nn-1,nn-2,wert2);
           s:=abs(wert1)+abs(wert2);
           y:=0.75*s;
           x:=0.75*s;
           w:=-0.4375*s*s;
          end;
         inc(its);
         for m:=(nn-2) downto 0 do
          begin
           b^.ReadMatrixElement(m,m,z);
           r:=x-z;
           s:=y-z;
           b^.ReadMatrixElement(m+1,m,wert1);
           b^.ReadMatrixElement(m,m+1,wert2);
           p:=(r*s-w)/wert1+wert2;
           b^.ReadMatrixElement(m+1,m+1,wert);
           q:=wert-z-r-s;
           b^.ReadMatrixElement(m+2,m+1,r);
           s:=abs(p)+abs(q)+abs(r);
           p:=p/s;
           q:=q/s;
           r:=r/s;
           if m=l then break;
           b^.ReadMatrixElement(m,m-1,wert);
           u:=abs(wert)*(abs(q)+abs(r));
           b^.ReadMatrixElement(m-1,m-1,wert1);
           b^.ReadMatrixElement(m+1,m+1,wert2);
           v:=abs(p)*(abs(wert1)+abs(z)+abs(wert2));
           if (u+v)=v then break;
          end;
         for i:=m+2  to nn do
          begin
           b^.WriteMatrixElement(i,i-2,0);
           if (i<>m+2) then b^.WriteMatrixElement(i,i-3,0);
          end;
         for k:=m to nn-1 do
          begin
           if k<>m then
            begin
             b^.ReadMatrixElement(k,k-1,p);
             b^.ReadMatrixElement(k+1,k-1,q);
             r:=0;
             if k<>nn-1 then b^.ReadMatrixElement(k+2,k-1,r);
             x:=abs(p)+abs(q)+abs(r);
             if x<>0 then
              begin
               p:=p/x;
               q:=q/x;
               r:=r/x;
              end;
            end;
           s:=sign(sqrt(p*p+q*q+r*r),p);
           if s<>0 then
            begin
             if k=m then
              begin
               if l<>m then
                begin
                 b^.ReadMatrixElement(k,k-1,wert);
                 b^.WriteMatrixElement(k,k-1,-wert);
                end;
              end
             else
              begin
               b^.WriteMatrixElement(k,k-1,-s*x);
              end;
             p:=p+s;
             x:=p/s;
             y:=q/s;
             z:=r/s;
             q:=q/p;
             r:=r/p;
             for j:=k to nn do
              begin
               b^.ReadMatrixElement(k,j,wert1);
               b^.ReadMatrixElement(k+1,j,wert2);
               p:=wert1+q*wert2;
               if k<>nn-1 then
                begin
                 b^.ReadMatrixElement(k+2,j,wert);
                 p:=p+r*wert;
                 b^.WriteMatrixElement(k+2,j,wert-p*z);
                end;
               b^.WriteMatrixElement(k+1,j,wert2-p*y);
               b^.WriteMatrixElement(k,j,wert1-p*x);
              end;
             if nn<k+3 then mmin:=nn
                       else mmin:=k+3;
             for j:=l to mmin do
              begin
               b^.ReadMatrixElement(i,k,wert1);
               b^.ReadMatrixElement(i,k+1,wert2);
               p:=x*wert1+y*wert2;
               if k<>nn-1 then
                begin
                 b^.ReadMatrixElement(i,k+2,wert);
                 p:=p+z*wert;
                 b^.WriteMatrixElement(i,k+2,wert-p*r);
                end;
               b^.WriteMatrixElement(i,k+1,wert2-p*q);
               b^.WriteMatrixElement(i,k,wert1-p);
              end;
            end;
          end;
        end;
      end;
   until (l<nn-2);
  end;
 b^.Done;
end;

function Matrix_Active:word;
var ch:char;
    i:integer;
begin
 write('Noch ',NoOfMatrices);
 if NoOfMatrices=1 then write(' Matrix ') else write(' Matrizen ');
 writeln('im Speicher: ');
 HighVideo;
 for i:=NoOfMatrices-1 downto 0 do
  begin
   write(AllMatrices[i]^.name);
   if i>0 then write(', ');
  end;
 writeln;
 NormVideo;
end;


function Matrix_Error(i:word;header:string):word;
(*******************************************)
(* INTERNAL USE ONLY                       *)
(*******************************************)
begin
 If AbortOnMatrixError then
  begin
   write(header);
   write(' (',MatrixErrorString(i),')');
   halt;
  end;
 Matrix_Error:=i;
end;


function MatrixErrorString(i:word):string;
begin
 case i of
   0: MatrixErrorString:='No Error detected';
   1: MatrixErrorString:='Out of Range';
   2: MatrixErrorString:='Must be a square matrix';
   3: MatrixErrorString:='Matrices must have same size';
   4: MatrixErrorString:='Row/Col different';
   5: MatrixErrorString:='Matrix is singular';
   6: MatrixErrorString:='Must be an vector';
   7: MatrixErrorString:='Vector doesnt match matrix';
   8: MatrixErrorString:='Grabbed matrix exceeds limits';
   9: MatrixErrorString:='To big to display';
  10: MatrixErrorString:='To many iterations';
  11: MatrixErrorString:='Must have same amount of elements';
 end;
end;

{$F+} procedure MatrixExitProcedure; {$F-}
var ch:char;
    i:integer;
begin
 if NoOfMatrices<>0 then
  begin
   writeln('--------------------------- Warning -------------------------');
   write('Noch ',NoOfMatrices);
   if NoOfMatrices=1 then write(' Matrix ') else write(' Matrizen ');
   writeln('im Speicher: ');
   HighVideo;
   for i:=NoOfMatrices-1 downto 0 do
    begin
     write(AllMatrices[i]^.name);
     if i>0 then write(', ');
    end;
   NormVideo;
   writeln;
   writeln('Press space to remove, any other key cancels');
   ch:=readkey;
   if ch=' ' then
    begin
     for i:=NoOfMatrices-1 downto 0 do
      begin
       Dispose(AllMatrices[i],Done);
       write('.');
      end;
    end;
   writeln(' - All Matrices deleted');
  end;
 ExitProc:=SaveExitProc;
end;

begin
 NoOfMatrices:=0;
 SaveExitProc:=ExitProc;
 ExitProc:=@MatrixExitProcedure;
end.